che <- "C:/Users/Jerome/Desktop/tds/master2_2021/donneescodes"
library(tidyverse)
library(lubridate)

conso <- read_tsv(file.path(che,'Donnees/historique_consommation_puissance_2003.txt')) %>%
  pivot_longer(-Date,names_to = "heure", values_to = "conso") %>%
  mutate(datim = dmy_hm(paste(Date, heure))) %>%
  select(-Date,-heure) %>%
  arrange(datim)

temp <- read_tsv(file.path(che,'Donnees/TREA03L_V1_040419.txt')) %>%
  pivot_longer(-Date,names_to = "heure", values_to = "temp") %>%
  mutate(datim = as.POSIXct(dmy(Date)) + minutes(30*as.numeric(heure)-60)) %>%
  select(-Date,-heure) %>%
  arrange(datim)

normaux <- left_join(conso, temp, by = "datim") %>%
  mutate(date = as_date(datim),
         temp = approxfun(datim,temp)(datim))
normaux2 <- left_join(conso, temp, by = "datim") %>%
  mutate(date = as_date(datim),
         temp = splinefun(datim,temp)(datim))

(bind_rows(normaux, normaux2, .id = "typapprox") %>%
  ggplot(aes(x = datim, y = temp, col = typapprox)) +
  geom_line()) %>% plotly::ggplotly(dynamicTicks = TRUE)


speciaux <- read_tsv(file.path(che,'Donnees/speciaux.txt')) %>%
  mutate(date = dmy(Date)) %>%
  select(-Date)

seuilchau <- 14
seuilclim <- 20

tout0 <- left_join(normaux, speciaux, by = "date") %>%
  mutate(
    fetenum = as.numeric(!(is.na(fete) | fete == 'non')),
    type = (wday(date)) * (1 - fetenum),
    ejp = ifelse(is.na(ejp), 0, ejp)
  ) %>%
  filter(date > dmy(01012003) & date < dmy(31122003))

OrdreF <- 3
tout <- tout0 %>%
  group_by(date(datim)) %>%
  mutate(tli = mean(temp)) %>%
  ungroup() %>%
  mutate(
    chau = pmax(0, seuilchau - temp),
    chaul = pmax(0, seuilchau - tli),
    clim = pmax(0, temp - seuilclim),
    h = 2 * hour(datim) + minute(datim) / 30,
    posans = matrix(ncol= 1,as.numeric(date) / 365) %*% matrix(nrow = 1, 1:OrdreF),
    a = cos(2 * pi * posans),
    b = sin(2 * pi * posans)
  ) %>%
  select(-posans)


system.time(
  prevconso <-
    lm(conso ~ 
         (as.factor(type) + a + b + chau + chaul + ejp):as.factor(h)
       + clim + as.numeric(date)*chaul,
       data = tout
    ))
spv <- summary(prevconso)
spv$sigma

errjoursfun <- function(abers){
  prevconso <-
    lm(conso ~ 
         (as.factor(type) + a + b + chau + chaul + ejp):as.factor(h)
       + clim + as.numeric(date),
       weights = as.numeric(!(date(tout$datim) %in% abers)),
       data = tout )
  spv <- summary(prevconso)
  print(spv$sigma)
  ####
  errjours <- cbind(resid(prevconso),tout)[!(date(tout$datim) %in% abers),] %>%
    rename(res=`resid(prevconso)`) %>%
    group_by(date=date(datim)) %>%
    summarise(rmse=sqrt(mean(res^2)), .groups = "drop") %>%
    arrange(-rmse) %>%
    mutate(rmsecum = cumsum(rmse))
  ####
  plot(errjours$rmse)
  plot(errjours$rmse[1:40])
  errjours
}

ruptures <- union(seq(dmy("010803"),dmy("150803"),by="day"),
                  seq(dmy("241203"),dmy("311203"),by="day"))
abers <- ruptures
errjours <- errjoursfun(abers)
errjours$date[1:2]
abers <- union(abers,unlist(errjours$date[1:2]))

errjours <- errjoursfun(abers)
errjours$date[1]
abers <- union(abers,unlist(errjours$date[1]))

errjours <- errjoursfun(abers)
errjours$date[1:13]
abers <- union(abers,unlist(errjours$date[1:13]))

errjours <- errjoursfun(abers)
errjours$date[1:5]
abers <- union(abers,unlist(errjours$date[1:5]))

errjours <- errjoursfun(abers)
as_date(sort(setdiff(abers,ruptures)))
