%let che = /home/jerome.collet1/Master19 ;

/* lectures */
data temperature;
  infile "&che/Donnees/TREA03L_V1_040419.txt" dlm='09'x firstobs=2;
  input date ddmmyy10. t1-t8;
  format date date.;
run;

data calendar;
  infile "&che/Donnees/speciaux.txt" dlm='09'x firstobs=2;
  length fete $25.;
  input date ddmmyy10. ejp fete;
  format date date.;
run;

data consommation;
  infile "&che/Donnees/historique_consommation_puissance_2003.txt" dlm='09'x firstobs=2 lrecl=300;
  input date ddmmyy10. conso1-conso48;
  format date date.;
run;

/* transpositions */
data temperaturecol; set temperature;
array tabtemp t1-t8;
do i=1 to 8;
  temperature=tabtemp(i);
  h=3*i;
  datim=dhms(date,h,0,0);
  output;
end;
drop t1-t8 i h;
format datim datetime.;
run;

data consocol; set consommation;
array contemp conso1-conso48;
do i=1 to 48;
  consommation=contemp(i);
  datim=dhms(date,0,30*i,0);
  output;
end;
drop conso1-conso48 i;
format datim datetime.;
run;

/* boucher les trous */
proc expand data=temperaturecol out=tempcolhor factor=6 method=spline;
  id datim;
  convert temperature;
run;

/* fusionner */
data consotemp; merge consocol tempcolhor;
  by datim;
  date=datepart(datim);
run;

proc sort data=calendar;
  by date;
run;

data consotempcal; merge consotemp calendar;
  by date;
  if date ne "1jan2003"d;
run;

proc datasets nolist nodetails;
  delete consommation consocol conscolplein temperature temperaturecol tempcolhor
  consotemp calendar;
run;quit;


/* modelisation */
%let PI = 3.14159265359;
%let li=0.003;
%let schau=15;
%let sclim=19;

data jeuaux; set consotempcal;
  if date>"30mar2003"d and date<"26oct2003"d then cheure=1; else cheure=0;
  date=datepart(datim);
  h=timepart(datim)/3600;
  posan=(date-mdy(12,21,year(date)))/365.25;
  d=date/365.25-35;
  if _n_<(10*24) then poids=0;
  retain tli 5;
  tli=(1-&li)*tli+&li*temperature;
  array a a1-a4;
  array b b1-b4;
  do i=1 to 4;
    a(i) = cos(2*&PI*i*posan);
    b(i) = sin(2*&PI*i*posan);
  end;
  drop i;
run;

data jeuaux; set jeuaux;
  poids=1;
  if ejp ne 1 then ejp=0;
  if fete = "" then type=weekday(date);
    else type=8;
  tlish=max(0,&schau-tli);
  tsh=max(0,&schau-temperature);
  tlisc=max(0,tli-&sclim);
  tsc=max(0,temperature-&sclim);
run;


ods select FitStatistics SelectedEffects NObs;
proc glmselect data=jeuaux;
  weight poids;
  class type h cheure ejp;
  model consommation =
    type*h h*cheure type*cheure ejp*h
    a1*h*cheure a2*h a3*h b1*h*cheure b2*h b3*h
    tsh*h tlish*h
    d*tsh d*tlish 
    tlisc*h
    d;
  output out=ventilo p=ventilo r=res_ventilo;
run;
quit;

%macro glm_rmse;
/* GLMselect, on ne garde qu'une partie des resultats, sinon trop longs */
ods select FitStatistics SelectedEffects NObs;
proc glmselect data=jeuaux;
  weight poids;
  class type h cheure ejp;
  model consommation =
    type*h h*cheure type*cheure ejp*h
    a1*h*cheure a2*h a3*h b1*h*cheure b2*h b3*h
    tsh*h tlish*h
    d*tsh d*tlish 
    tlisc*h
    d;
  output out=ventilo p=ventilo r=res_ventilo;
run;
quit;
/* calcul rmse */
data ventilo; set ventilo;
  se=res_ventilo**2;
run;
proc summary data=ventilo;
  where poids=1;
  class date;
  output out=rmsejour mean(se)=mse;
run;
data rmsejour; set rmsejour;
  rmse=sqrt(mse);
  drop mse;
run;
proc sort data=rmsejour;
  by descending rmse;
run;
data rmsejour; set rmsejour;
  ordre=_n_;
run;
proc print data=rmsejour;
  var date rmse;
  where ordre<25;
run;
%mend;

%glm_rmse;

data jeuaux; set jeuaux;
  /* on enleve les 10 jours les plus mal prevus, qui sont tous particuliers, voir un calendrier */
  if date in ("02JAN03"d "26DEC03"d "10NOV03"d "24DEC03"d "30MAY03"d "31DEC03"d "03JAN03"d "30DEC03"d "02MAY03"d)
    then poids=0;
run;
%glm_rmse;

data jeuaux; set jeuaux;
  if date in ("09JUN03"d "08JUN03"d) then poids=0;
  if date>="01AUG03"d and date<="22AUG03"d then poids=0;
  if date>"22DEC03"d then poids=0;
run;
%glm_rmse;

data jeuaux; set jeuaux;
  if date in ("09May03"d "10JUN03"d) then poids=0;
run;
%glm_rmse;


