/* Statistiques descriptives elementaires avec SAS */

/* Biliotheque de tables SAS ltables dans le repertoire 'tables' */
%let che=/home/jerome.collet1/Master19/Tables;
libname ltables "&che";

/* Copie de la table loadcal de la bibliotheque ltables vers work */
data loadcal; set ltables.loadcal;
run;

/* Statistiques descriptives */

/* Statistiques univariees sur variable quantitative */
proc univariate data=loadcal normal;
  var load;
run;

ods graphics off;
ods select ParameterEstimates GoodnessOfFit FitQuantiles MonDessin;
proc univariate data=loadcal;
  var load;
  histogram / lognormal weibull gamma name= 'MonDessin';
run;

/* Box plot */
proc sort data=loadcal out=loadcal_d;
  by daytype;
run;

proc boxplot data=loadcal_d;
  plot load*daytype;
run;

/* Tableaux de frequence */
/* La proc freq permet de tester des hypotheses de distribution */
proc freq data=loadcal;
  table offset;
run;

/* correlations */
proc corr data=loadcal;
run;

/* On selectione les observations interessantes et on trie par wday
   L'option keep= permet de ne garder que certaines variables
 */

data loadcal_w;
  set loadcal;
  wday = weekday(date);
  if wday in (3 6) and daytype = 1;
  keep wday load hour date;
run;

/* On teste la normalit� de q0 par jour de la semaine
 */
proc univariate data=loadcal_w normal;
  var load;
  class wday;
  histogram;
run;
/* La p-Value est << 0.05 : on refuse "H0 : load est normal" */
/* Mais le test de student est robuste :
   bcp d'obs, ne pas se pr�ocuper des lois ni des variances */

/* comparaisons des distributions par wday */
proc ttest data=loadcal_w;
  class wday;
  var load;
run;

proc ttest data=loadcal_w;
  where hour=2;
  class wday;
  var load;
run;
proc ttest data=loadcal_w;
  where hour=17;
  class wday;
  var load;
run;

proc summary data=loadcal_w nway;
  class date wday;
  output out=loadcalwjour sum(load)=;
run;
proc ttest data=loadcalwjour;
  class wday;
  var load;
run;
